/* Name: main.c
 * Project: Thermostat based on AVR USB driver
 * Author: Christian Starkjohann
 * Creation Date: 2006-04-23
 * Tabsize: 4
 * Copyright: (c) 2006 by OBJECTIVE DEVELOPMENT Software GmbH
 * License: Proprietary, free under certain conditions. See Documentation.
 * This Revision: $Id: main.c 537 2008-02-28 21:13:01Z cs $
 */

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>

#include "usbdrv.h"
#include "oddebug.h"
#include "I2C.h"


PROGMEM char usbHidReportDescriptor[22] = {    /* USB report descriptor */
    0x06, 0x00, 0xff,              // USAGE_PAGE (Generic Desktop)
    0x09, 0x01,                    // USAGE (Vendor Usage 1)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x26, 0xff, 0x00,              //   LOGICAL_MAXIMUM (255)
    0x75, 0x08,                    //   REPORT_SIZE (8)
    0x95, 16,                    //   REPORT_COUNT (16)
    0x09, 0x00,                    //   USAGE (Undefined)
    0xb2, 0x02, 0x01,              //   FEATURE (Data,Var,Abs,Buf)
    0xc0                           // END_COLLECTION
};
/* Since we define only one feature report, we don't use report-IDs (which
 * would be the first byte of the report). The entire report consists of 128
 * opaque data bytes.
 */

/* The following variables store the status of the current data transfer */
static uchar    currentAddress;
static uchar    bytesRemaining;

static uint8_t tempbuff[16] = {0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1}; 
static uint8_t timer = 0;

void ReadTemps() {
	for(uint8_t x = 0; x < 8; x++) {
		if(I2C_Start(I2C_DS75_BASE + x, I2C_READ) == 0) {
			tempbuff[x*2] = 0;
			tempbuff[x*2+1] = 1; //not found
		} else {
			tempbuff[x*2] = I2C_Read(0);
			tempbuff[x*2+1] = I2C_Read(1);
		}
		I2C_Stop();
	}
}

void SetRes() {
	for(uint8_t x = 0; x < 8; x++) {
		if(I2C_Start(I2C_DS75_BASE + x, I2C_WRITE) == 0) {
			I2C_Stop();
		} else {
			I2C_Write(1); //Pointer auf Configure
			I2C_Write(0x60); //12 Bit Resolution
			I2C_Stop();
			I2C_Start(I2C_DS75_BASE + x, I2C_WRITE);
			I2C_Write(0); //Pointer auf Temperature
			I2C_Stop(); 
		}
	}
}


/* ------------------------------------------------------------------------- */
/* ------------------------ interface to USB driver ------------------------ */
/* ------------------------------------------------------------------------- */

/* ------------------------------------------------------------------------- */

/* usbFunctionRead() is called when the host requests a chunk of data from
 * the device. For more information see the documentation in usbdrv/usbdrv.h.
 */
 
uchar   usbFunctionRead(uchar *data, uchar len)
{
    if(len > bytesRemaining)
        len = bytesRemaining;

	for(uint8_t x = 0; x < len; x++) {
		data[x] = tempbuff[currentAddress + x];
		
	}

    currentAddress += len;
    bytesRemaining -= len;
	if(bytesRemaining == 0) timer = 255;

    return len;
}

/* usbFunctionWrite() is called when the host sends a chunk of data to the
 * device. For more information see the documentation in usbdrv/usbdrv.h.
 */

uchar   usbFunctionWrite(uchar *data, uchar len)
{
    if(bytesRemaining == 0)
        return 1;               // end of transfer
    if(len > bytesRemaining)
        len = bytesRemaining;
    currentAddress += len;
    bytesRemaining -= len;
    return bytesRemaining == 0; //return 1 if this was the last chunk
}

/* ------------------------------------------------------------------------- */

usbMsgLen_t usbFunctionSetup(uchar data[8]) {
usbRequest_t *rq = (void *)data;

    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){    /* HID class request */
        if(rq->bRequest == USBRQ_HID_GET_REPORT){  /* wValue: ReportType (highbyte), ReportID (lowbyte) */
            /* since we have only one report type, we can ignore the report-ID */
            bytesRemaining = 16;
            currentAddress = 0;
			PORTB &= ~(1<<PB0); //LED an
			ReadTemps();
			
			return USB_NO_MSG;

            //return USB_NO_MSG;  /* use usbFunctionRead() to obtain data */
        }else if(rq->bRequest == USBRQ_HID_SET_REPORT){
            // since we have only one report type, we can ignore the report-ID 
            bytesRemaining = 16;
            currentAddress = 0;
			PORTB &= ~(1<<PB0); //LED an
			
			return USB_NO_MSG; /* use usbFunctionWrite() to receive data from host */

        }
    }else{
        /* ignore vendor type requests, we don't use any */
    }
    return 0;
}

/* ------------------------------------------------------------------------- */
/* --------------------------------- main ---------------------------------- */
/* ------------------------------------------------------------------------- */

int main(void)
{
uchar   i;
	
	DDRB |= (1<<PB0);
	PORTB &= ~(1<<PB0); //LED an

    wdt_enable(WDTO_2S);
	I2C_Init();
    /* Even if you don't use the watchdog, turn it off here. On newer devices,
     * the status of the watchdog (on/off, period) is PRESERVED OVER RESET!
     */
    DBG1(0x00, 0, 0);       /* debug output: main starts */
    /* RESET status: all port bits are inputs without pull-up.
     * That's the way we need D+ and D-. Therefore we don't need any
     * additional hardware initialization.
     */
    odDebugInit();
    usbInit();
    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
    i = 0;
    while(--i){             /* fake USB disconnect for > 250 ms */
        wdt_reset();
        _delay_ms(1);
    }

	PORTB |= (1<<PB0); //LED aus
    usbDeviceConnect();
    sei();
	SetRes();
    for(;;){    /* main event loop */
			DBG1(0x02, 0, 0);
			wdt_reset();
        usbPoll();
		if(timer > 0) {
			timer--;
			if(timer == 0) PORTB |= (1<<PB0);
		}
			
    }
    return 0;
}

static void calibrateOscillator(void)
{
uchar       step = 128;
uchar       trialValue = 0, optimumValue;
int         x, optimumDev, targetValue = (unsigned)(1499 * (double)F_CPU / 10.5e6 + 0.5);

    /* do a binary search: */
    do{
        OSCCAL = trialValue + step;
        x = usbMeasureFrameLength();    /* proportional to current real frequency */
        if(x < targetValue)             /* frequency still too low */
            trialValue += step;
        step >>= 1;
    }while(step > 0);
    /* We have a precision of +/- 1 for optimum OSCCAL here */
    /* now do a neighborhood search for optimum value */
    optimumValue = trialValue;
    optimumDev = x; /* this is certainly far away from optimum */
    for(OSCCAL = trialValue - 1; OSCCAL <= trialValue + 1; OSCCAL++){
        x = usbMeasureFrameLength() - targetValue;
        if(x < 0)
            x = -x;
        if(x < optimumDev){
            optimumDev = x;
            optimumValue = OSCCAL;
        }
    }
    OSCCAL = optimumValue;
}


void    usbEventResetReady(void)
{
    calibrateOscillator();
}

